@@ -1,6 +1,7 @@
 """
 Threshold Graphs - Creation, manipulation and identification.
 """
+
 from math import sqrt
 
 import networkx as nx
@@ -9,6 +10,7 @@ from networkx.utils import py_random_state
 __all__ = ["is_threshold_graph", "find_threshold_graph"]
 
 
+@nx._dispatchable
 def is_threshold_graph(G):
     """
     Returns `True` if `G` is a threshold graph.
@@ -37,7 +39,7 @@ def is_threshold_graph(G):
     ----------
     .. [1] Threshold graphs: https://en.wikipedia.org/wiki/Threshold_graph
     """
-    return is_threshold_sequence(list(d for n, d in G.degree()))
+    return is_threshold_sequence([d for n, d in G.degree()])
 
 
 def is_threshold_sequence(degree_sequence):
@@ -300,6 +302,7 @@ def weights_to_creation_sequence(
 
 
 # Manipulating NetworkX.Graphs in context of threshold graphs
+@nx._dispatchable(graphs=None, returns_graph=True)
 def threshold_graph(creation_sequence, create_using=None):
     """
     Create a threshold graph from the creation sequence or compact
@@ -351,13 +354,14 @@ def threshold_graph(creation_sequence, create_using=None):
     return G
 
 
+@nx._dispatchable
 def find_alternating_4_cycle(G):
     """
     Returns False if there aren't any alternating 4 cycles.
     Otherwise returns the cycle as [a,b,c,d] where (a,b)
     and (c,d) are edges and (a,c) and (b,d) are not.
     """
-    for (u, v) in G.edges():
+    for u, v in G.edges():
         for w in G.nodes():
             if not G.has_edge(u, w) and u != w:
                 for x in G.neighbors(w):
@@ -366,6 +370,7 @@ def find_alternating_4_cycle(G):
     return False
 
 
+@nx._dispatchable(returns_graph=True)
 def find_threshold_graph(G, create_using=None):
     """
     Returns a threshold subgraph that is close to largest in `G`.
@@ -390,7 +395,7 @@ def find_threshold_graph(G, create_using=None):
     >>> from networkx.algorithms.threshold import find_threshold_graph
     >>> G = nx.barbell_graph(3, 3)
     >>> T = find_threshold_graph(G)
-    >>> T.nodes # may vary
+    >>> T.nodes  # may vary
     NodeView((7, 8, 5, 6))
 
     References
@@ -400,6 +405,7 @@ def find_threshold_graph(G, create_using=None):
     return threshold_graph(find_creation_sequence(G), create_using)
 
 
+@nx._dispatchable
 def find_creation_sequence(G):
     """
     Find a threshold subgraph that is close to largest in G.
@@ -569,7 +575,7 @@ def shortest_path(creation_sequence, u, v):
     u and v must be integers in (0,len(sequence)) referring
     to the position of the desired vertices in the sequence.
 
-    For a labeled creation_sequence, u and v are labels of veritices.
+    For a labeled creation_sequence, u and v are labels of vertices.
 
     Use cs=creation_sequence(degree_sequence,with_labels=True)
     to convert a degree sequence to a creation sequence.
@@ -663,7 +669,7 @@ def betweenness_sequence(creation_sequence, normalized=True):
     """
     Return betweenness for the threshold graph with the given creation
     sequence.  The result is unscaled.  To scale the values
-    to the iterval [0,1] divide by (n-1)*(n-2).
+    to the interval [0,1] divide by (n-1)*(n-2).
     """
     cs = creation_sequence
     seq = []  # betweenness
@@ -674,7 +680,7 @@ def betweenness_sequence(creation_sequence, normalized=True):
     dlast = 0.0  # betweenness of last d
     for i, c in enumerate(cs):
         if c == "d":  # cs[i]=="d":
-            # betweennees = amt shared with eariler d's and i's
+            # betweenness = amt shared with earlier d's and i's
             #             + new isolated nodes covered
             #             + new paths to all previous nodes
             b = dlast + (irun - 1) * irun / dr + 2 * irun * (i - drun - irun) / dr
@@ -825,7 +831,7 @@ def random_threshold_sequence(n, p, seed=None):
     """
     Create a random threshold sequence of size n.
     A creation sequence is built by randomly choosing d's with
-    probabiliy p and i's with probability 1-p.
+    probability p and i's with probability 1-p.
 
     s=nx.random_threshold_sequence(10,0.5)
 
